<?php

/**
 * A jQuery show/hide for categories and products.
 *
 * @package WordPress
 * @subpackage sjf-et
 * @since SJF ET 0.1
 */

function sjf_et_register_accordion() {
	register_widget( 'SJF_ET_Accordion' );
}
add_action( 'widgets_init', 'sjf_et_register_accordion' );

/**
 * Adds SJF_ET_Show_Hide_Products_Widget widget.
 */
class SJF_ET_Accordion extends WP_Widget {

	/**
	 * Register SJF_ET_Show_Hide_Products_Widget with WordPress.
	 */
	function __construct() {
		
		// Every widget has a shortcode version.
		add_shortcode( SJF_Ecwid_Formatting::get_class_name( __CLASS__ ), array( $this, 'shortcode' ) );

		// Add our docs to the admin docs section.
		add_filter( 'SJF_Ecwid_Admin_Documentation_get_docs', array( $this, 'get_docs' ), 20 );

		parent::__construct(

			// Base ID.
			SJF_Ecwid_Formatting::get_class_name( __CLASS__ ),

			// Name.
			sprintf( __( '%s: Accordion', 'sjf-et' ), SJF_Ecwid_Helpers::get_plugin_short_title() ),

			// Args.
			array(
				'description' => __( 'A list of products, by category, as an accordion show/hide.', 'sjf-et' ),
			)
		);

	}

	/**
	 * Send the widget as a shortcode.
	 * 
	 * @param  array $atts An array of shortcode args.
	 * @return string      The widget html output.
	 */
	public function shortcode( $atts ) {
		
		$base_class = SJF_Ecwid_Formatting::get_class_name( __CLASS__ );
		$base_id    = SJF_Ecwid_Formatting::get_class_name( __CLASS__ . '_' . rand() );

		$args = shortcode_atts( array(
			'title'             => '',
			'before_title'      => "<h3 class='$base_class-title'>",
			'after_title'       => '</h3>',
			'before_widget'     => "<div id='$base_id' class='$base_class-wrapper'>",
			'after_widget'      => '</div>',
			'widget_id'         => $base_id,
			'order'             => '',
			'order_by'          => '',
			'enabled'           => '',
			'hidden_categories' => '',
		), $atts, __CLASS__ );
	
		$instance['title']= $args['title'];

		$out = $this -> widget( $args, $instance, FALSE );

		$out = apply_filters( __CLASS__ . '_' . __FUNCTION__, $out );

		return $out;

	}

	/**
	 * Front-end display of widget.
	 *
	 * @see WP_Widget::widget()
	 *
	 * @param array $args     Widget arguments.
	 * @param array $instance Saved values from database.
	 */
	public function widget( $args = array(), $instance = array(), $echo = TRUE ) {
	
		// We're gonna need dashicons.
		wp_enqueue_style( 'dashicons' );
	
		// This specific widget instance.  Important for JS actions targeted specifically to this instance.
		$widget_id = esc_attr( $args['widget_id'] );
		
		$before_widget = '';
		if( isset( $args['before_widget'] ) ) {
			$before_widget = $args['before_widget'];
		}

		$after_widget = '';
		if( isset( $args['after_widget'] ) ) {
			$after_widget  = $args['after_widget'];
		}

		$title = '';
		if ( ! empty( $instance['title'] ) ) {
			$before_title = $args['before_title'];
			$after_title  = $args['after_title'];
			$title        = $before_title . apply_filters( 'widget_title', $instance['title'] ) . $after_title;
		}

		if ( ! empty( $instance['order'] ) ) {
			$args['order'] = $instance['order'];
		}	

		if ( ! empty( $instance['order_by'] ) ) {
			$args['order_by'] = $instance['order_by'];
		}

		if ( ! empty( $instance['enabled'] ) ) {
			$args['enabled'] = $instance['enabled'];
		}		

		if ( ! empty( $instance['hidden_categories'] ) ) {
			$args['hidden_categories'] = $instance['hidden_categories'];
		}

		// Grab the holder div into which the widget will ajax-load.	
		$load   = SJF_Ecwid_Ajax::get_load_el( $widget_id );

		// Grab a tiny bit of JS to ajax-load the widget.
		$script = SJF_Ecwid_Ajax::get_script( __CLASS__, $widget_id, $args );

		$out = $before_widget . $title . $load . $after_widget . $script;

		if( $echo ) {
			echo $out;
		} else {
			return $out;
		}

	}

	/**
	 * Back-end widget form.
	 *
	 * @see WP_Widget::form()
	 *
	 * @param array $instance Previously saved values from database.
	 */
	public function form( $instance ) {
		
		echo SJF_Ecwid_Helpers::get_nag();

		$title             = $instance['title'];
		$order_by          = $instance['order_by'];
		$order             = $instance['order'];
		$enabled           = $instance['enabled'];
		$hidden_categories = $instance['hidden_categories'];

		?>
		
		<p>
			<label for="<?php echo $this->get_field_id( 'title' ); ?>"><?php echo esc_html__( 'Title:', 'sjf-et' ); ?></label> 
			<input class="widefat" id="<?php echo $this->get_field_id( 'title' ); ?>" name="<?php echo $this->get_field_name( 'title' ); ?>" type="text" value="<?php echo esc_attr( $title ); ?>">
		</p>
		
		<p>
			<label for="<?php echo $this->get_field_id( 'order_by' ); ?>"><?php echo esc_html__( 'Order By:', 'sjf-et' ); ?></label> 
			<select class="widefat" id="<?php echo $this->get_field_id( 'order_by' ); ?>" name="<?php echo $this->get_field_name( 'order_by' ); ?>" >
				<?php echo SJF_Ecwid_Admin_Helpers::get_orderby_as_options( $order_by ); ?>
			</select>
		</p>

		<p>
			<label for="<?php echo $this->get_field_id( 'order' ); ?>"><?php echo esc_html__( 'Order:', 'sjf-et' ); ?></label> 
			<select class="widefat" id="<?php echo $this->get_field_id( 'order' ); ?>" name="<?php echo $this->get_field_name( 'order' ); ?>" >
				<?php echo SJF_Ecwid_Admin_Helpers::get_order_as_options( $order ); ?>
			</select>
		</p>	

		<p>
			<label for="<?php echo $this->get_field_id( 'enabled' ); ?>"><?php echo esc_html__( 'Product Availability:', 'sjf-et' ); ?></label> 
			<select class="widefat" id="<?php echo $this->get_field_id( 'enabled' ); ?>" name="<?php echo $this->get_field_name( 'enabled' ); ?>" >
				<?php echo SJF_Ecwid_Admin_Helpers::get_enabled_as_options( $enabled ); ?>
			</select>
		</p>

		<p>
			<label for="<?php echo $this->get_field_id( 'hidden_categories' ); ?>"><?php echo esc_html__( 'Category Availability:', 'sjf-et' ); ?></label> 
			<select class="widefat" id="<?php echo $this->get_field_id( 'hidden_categories' ); ?>" name="<?php echo $this->get_field_name( 'hidden_categories' ); ?>" >
				<?php echo SJF_Ecwid_Admin_Helpers::get_hidden_categories_as_options( $hidden_categories ); ?>
			</select>
		</p>		

		<?php 
	
	}

	/**
	 * Sanitize widget form values as they are saved.
	 *
	 * @see WP_Widget::update()
	 *
	 * @param array $new_instance Values just sent to be saved.
	 * @param array $old_instance Previously saved values from database.
	 *
	 * @return array Updated safe values to be saved.
	 */
	public function update( $new_instance, $old_instance ) {
		
		/**
		 * When the transients class is initiated with a value of FALSE (first arg), it dumps caches.
		 * However the second arg, FALSE, tells it not to dump rewrite rules, since that would break the
		 * widget customizer screen.
		 */
		$trans = new SJF_Ecwid_Transients( FALSE, FALSE );

		$instance = array();
		
		$instance['title']             = strip_tags( $new_instance['title'] );
		$instance['order']             = strip_tags( $new_instance['order'] );
		$instance['order_by']          = strip_tags( $new_instance['order_by'] );
		$instance['enabled']           = strip_tags( $new_instance['enabled'] );
		$instance['hidden_categories'] = strip_tags( $new_instance['hidden_categories'] );

		return $instance;
	}

	/**
	 * A recursive function to build a nested list of categories & products.
	 * 
	 * @param  mixed $parent_cat Restrict the list to those categories that are a child of a given category.
	 * @return string A nested list of categories & products.
	 */
	function get_html( $args = FALSE ) {

		// We're gonna need dashicons.
		wp_enqueue_style( 'dashicons' );

		$parent_cat = FALSE;
		if( isset( $args['parent_cat'] ) ) {
			$parent_cat = $args['parent_cat'];
		}

		$namespace = SJF_Ecwid_Helpers::get_namespace();

		$base_class = SJF_Ecwid_Formatting::get_class_name( __CLASS__ );

		$out = '';

		// If there is a parent cat, restrict the results to children of that cat.
		if( $parent_cat ) {
			$collection_args = array(
				'parent' => $parent_cat,
			);

		// Else, ecwid expects a value of 0 to grab all categories.
		} else {
			$collection_args = array(
				'parent' => 0,
			);
		}

		$enabled = FALSE;
		if( isset( $args['enabled'] ) ) {
			$enabled = $args['enabled'];
			$collection_args['enabled'] = $args['enabled'];
		}

		$hidden_categories = FALSE;
		if( isset( $args['hidden_categories'] ) ) {
			$hidden_categories = $args['hidden_categories'];
			$collection_args['hidden_categories'] = $args['hidden_categories'];
		}

		// Get the categories.
		$collection = new SJF_Ecwid_Collection( 'categories', $collection_args );
		$result = $collection -> get_collection();
		if( ! isset( $result['items'] ) ) {
			return FALSE;
		}

		$items = $result['items'];	
		if( ! is_array( $items ) ) {
			return FALSE;
		}	
		
		$order = FALSE;
		if( isset( $args['order'] ) ) {
			$order = $args['order'];
		}
		
		$order_by = FALSE;
		if( isset( $args['order_by'] ) ) {
			$order_by = $args['order_by'];
		}	

		$items = SJF_Ecwid_Helpers::sort( $items, $order, $order_by );

		// For each category...
		foreach( $items as $cat ) {

			$id   = absint( $cat['id'] );
			$href = esc_url( $cat['url'] );
			$name = esc_html( $cat['name'] );

			// Build the linked title.
			$linked_title = "<a class='$base_class-cat-title' href='$href'>$name</a>";
			$linked_title = apply_filters( __CLASS__ . '_' . __FUNCTION__ . '_linked_title', $linked_title, $cat );					

			// Grab child cats recursively.
			$this_args = array(
				'parent_cat'        => $id,
				'order'             => $order,
				'order_by'          => $order_by,
				'hidden_categories' => $hidden_categories,
			);
			$child_cats = $this -> get_html( $this_args );
			$child_cats = apply_filters( __CLASS__ . '_' . __FUNCTION__ . '_child_cats', $child_cats, $cat );					

			// Grab products that fall under this category.
			$products = $this -> get_products_list( $id, $order, $order_by, $enabled );
			$products = apply_filters( __CLASS__ . '_' . __FUNCTION__ . '_products', $products, $cat );					

			// If there are products or child cats, provide a link to show them.
			$toggle = '';
			if( ! empty( $products ) || ! empty( $child_cats ) ) {
	
				$toggle = SJF_Ecwid_Helpers::get_toggle( array( "$base_class-cat-toggle" ) );
				$toggle = apply_filters( __CLASS__ . '_' . __FUNCTION__ . '_toggle', $toggle, $cat );
			}

			// Output this category with all the sub cats and products nested under it.
			$this_cat = "
				<li class='$base_class-cat $namespace-toggle-parent'>
					$linked_title
					$toggle 
					$products 
					$child_cats
				</li>
			";
			$this_cat = apply_filters( __CLASS__ . '_' . __FUNCTION__ . '_cat', $this_cat, $cat );

			$out .= $this_cat;
			
		}

		if( ! empty( $out ) ) {

			// If this cat is udner a parent cat, hide it.
			if( $parent_cat ) {

				$out = "<ul class='$base_class-cats $base_class'>$out</ul>";

			} else {
	
				$out = "<ul class='$base_class-cats'>$out</ul>";

			}

		}

		$out = apply_filters( __CLASS__ . '_' . __FUNCTION__, $out );
		
		return $out;

	}	

	/**
	 * Get an HTML list of products from a given category.
	 * 
	 * @param  int $cat_id The category from which we'll grab products.
	 * @return string an HTML list of products from a given category.
	 */
	function get_products_list( $cat_id, $order, $order_by, $enabled ) {

		$out = '';

		$namespace = SJF_Ecwid_Helpers::get_namespace();

		$base_class = SJF_Ecwid_Formatting::get_class_name( __CLASS__ );

		// By default, we order them alphabetically.
		$sortBy = 'NAME_ASC';

		// We can sort them in ascending order by...
		if( $order == 'ASC' ) {

			// Date created.
			if( $order_by == 'date_created' ) {
				$sortBy = 'ADDED_TIME_ASC';
			}

		// And we can sort them in descending order by...
		} elseif( $order == 'DESC' ) {

			// Date created or...
			if( $order_by == 'date_created' ) {
				$sortBy = 'ADDED_TIME_DESC';
			
			// Name.
			} elseif( $order_by == 'name' ) {
				$sortBy = 'NAME_DESC';
			}

		}

		// Get the products.
		$args = array(
			'category' => $cat_id,
			'sortBy'   => $sortBy,
			'enabled'  => $enabled,
		);

		$collection = new SJF_Ecwid_Collection( 'products', $args );
		$result = $collection -> get_collection();
		if( ! is_array( $result ) ) {
			return FALSE;
		}
		if( ! isset( $result['items'] ) ) {
			return FALSE;
		}
		$items = $result['items'];

		// For each product...
		foreach( $items as $item ) {

			// Build a link to that product.
			$href = esc_url( $item['url'] );
			$name = esc_html( $item['name'] );
			$name = apply_filters( __CLASS__ . '_' . __FUNCTION__ . '_name', $name, $item );

			$product_link = "<a class='$base_class-prod-link' href='$href'>$name</a>";
			$product_link = apply_filters( __CLASS__ . '_' . __FUNCTION__ . '_link', $product_link, $item );

			$out .= "
				<li class='$base_class-prod $base_class-prod-item'>
					$product_link
				</li>
			";

		}

		if( ! empty( $out ) ) {
			$out = "<ul class='$base_class-prods $base_class'>$out</ul>";
			$out = apply_filters( __CLASS__ . '_' . __FUNCTION__, $out );
		}

		return $out;

	}

	/**
	 * Get info about the accordion shortcode.
	 * 
	 * @return string Info about the accordion shortcode.
	 */
	function get_docs( $in ) {

		$docs = new SJF_Ecwid_Admin_Documentation;

		$namespace = SJF_Ecwid_Helpers::get_namespace();

		$label = esc_html__( 'Accordion Shortcode', 'sjf-et' );
		
		$content_1 = '<p>' . esc_html__( 'The accordion shortcode can be used like this:', 'sjf-et') . '</p>'; 
		$content_2 = '<p><code>[sjf_et_accordion order_by="category_order" order="DESC" enabled="both" hidden_categories="false"]</code></p>';
		$content_3 = '<p>' . esc_html__( 'You may specify an `order` ("ASC" or "DESC").', 'sjf-et' ) . '</p>';
		$content_4 = '<p>' . esc_html__( 'You may specify a field for `order_by` ("name", "category_order", "product_count", or "date_created").', 'sjf-et' ) . '</p>';
		$content_5 = '<p>' . esc_html__( 'You may specify a value for `enabled` ("true", "false" or "both").', 'sjf-et' ) . '</p>';
		$content_6 = '<p>' . esc_html__( 'You may specify a value for `hidden_categories` ("true" or "false").', 'sjf-et' ) . '</p>'; 

		$content = $content_1 . $content_2 . $content_3 . $content_4 . $content_5 . $content_6;

		$out = $docs -> get_doc( $label, __FUNCTION__, $content );

		return $in . $out;

	}

}